// Processing Code fuer BME280 Wetterstation
// 27.06.2022
// F.G.


import processing.serial.*;

Serial myPort;  // Create object from Serial class
int val;      // Data received from the serial port

boolean ArduinoIsConnected = true;

Koordinatensystem ks; // Ein Objekt, dass das Koordinatensystem darstellen wird

int index = 1; // fuer die Kommunikation mit dem Arduino
char[] buffer = new char[20];

long timeStamp = millis();

int counter = 0;

void setup() {
  size(800, 400); // Leinwand einer bestimmten Groesse anfordern
  background(0,50,20);
  
  ks = new Koordinatensystem(width, height);  
  ks.setXRange(-50,750); // Wertebereiche angeben, die dargestellt werden sollen
  ks.setYRange(-50,50);
  //ks.setLimit(150); // Nach x geplotteten Punkten alles wieder loeschen
  ks.punkteLoeschen(); // Bildschirm 'saubermachen'; 
  ks.punktFarbe(0,255,0);
  //ks.druckePunkt(100.3,23.0); // Testpunkt plotten
  
  
  // Fuer die Kommunikation mit dem Arduino notwendig  
  // I know that the first port in the serial list on my mac
  // is always my  FTDI adaptor, so I open Serial.list()[0].
  // On Windows machines, this generally opens COM1.
  // Open whatever port is the one you're using.  
  println("Gefundene Geraete: ");
  printArray(Serial.list());
  try {
    String portName = Serial.list()[index];
    myPort = new Serial(this, portName, 9600);
    ArduinoIsConnected = true;
  } catch (Exception e) {
    ArduinoIsConnected = false;
    // Wenn der Arduino nicht angeschlossen ist, starte ein anderes Demo
    ks.setLimit(100);
    ks.setXRange(-500,500); // Wertebereiche angeben, die dargestellt werden sollen
    ks.setYRange(-500,500); // Wertebereiche angeben, die dargestellt werden sollen
    ks.punkteLoeschen();
  }

}

void draw() {
  
  if (ArduinoIsConnected) {
    // Auf den Arduino 'horchen'
    if ( myPort.available() > 0) {  // If data is available,
      int lf = 33; // Der Decimalwert von 33 entspricht einem ! in der ASCII Tabelle. Das ! Zeichen habe ich als Terminator gewählt
      // Expand array size to the number of bytes you expect:
      byte[] inBuffer = new byte[100];
      myPort.readBytesUntil(lf, inBuffer);
      if (inBuffer != null) {
        String myString = new String(inBuffer);
        //println("Diese Nachricht hat der Arduino geschickt: "+myString);
        verarbeiteNachricht(myString);
        }
      
    } // Ende 'horchen'
  } else {
    // Wenn kein Arduino angeschlossen ist, ein anderes Demo durchfuehren
    if (abs(millis()-timeStamp)>250) {
      ks.punktFarbe((int)random(255),(int)random(255),(int)random(255));
      ks.druckePunkt(random(-500,500), random(-500,500));
      timeStamp = millis();
    }
    
  }
  
} // Ende der Draw Methode

void verarbeiteNachricht(String s) {
  //Parsen. Der Adruino (Code ganz unten) schickt Nachrichten in der Form-->    Sensor1:94#135! 
  try {
    String s1 = s.substring(s.lastIndexOf(":")+1, s.lastIndexOf("#"));
    String s2 = s.substring(s.lastIndexOf("#")+1, s.lastIndexOf("!"));
    int s1_ = Integer.parseInt(s1);  
    int s2_ = Integer.parseInt(s2);
    int w1=s1_%width;      //sorgt für Umbruch am Rechten Bildschirmrand, 
                           // geht davon aus das in s1 ein Counter übertragen wird.
    println("Gesendete Koordinaten: "+w1+ " -- " + s2_);
   
    //ks.druckePunkt((w1),s2_);
    ks.punktFarbe(0,255,0);
    ks.druckePunkt(counter,s1_);
    ks.punktFarbe(0,205,255);
    ks.druckePunkt(counter,s2_);
    counter = counter +1;
    
    
  } catch (Exception e) {
  }
}


// ##########################################################################################
// #################  Ab hier sollten Sie nichts veraendern  ################################
// ##########################################################################################

// -------------------------------------------------------------
// Eine Klasse, um das Koordinatensystem graphisch darzustellen
class Koordinatensystem {
  int counter = 0; // Anzahl von Punkten die ins Koordinatensystem geplottet worden sind
  int limit = 0; // Anuzahl von Punkten, die in ein Koordinatensystem geplottet werden dürfen
  
  int window = 50; // Fenstergroesse der Skaliereung.
  
  //int nullX = 0;// Position des Nullpunktes des Koordinatensystems in Pixelkoordinaten
  //int nullY = 0;
  
  int b = 0; // Breite des Koordinatensystems
  int h = 0; // Hoehe
  
  float y_min = 0; // min max Wertem die im Koordinatensystem angezeigt werden
  float y_max = 300;
  float x_min = 0;
  float x_max = 400;  
  
  
  int r_ = 255;
  int g_ = 0;
  int b_ = 0;
  
  // Constructor; Moechte als Parameter 'wissen', wie groß die Leinwand ist (in Pixeln)
  Koordinatensystem(int breite, int hoehe) {
    b = breite;
    h = hoehe;
  }
  
  Koordinatensystem() {
    b = width;
    h = height;
    showAxes();
    showLegend();
  }
  

  // Bestimmen von wo bis wo die Y-Achse geht...
  void setYRange(float min_, float max_) {
    y_min = min_;
    y_max = max_;
  }
  // Bestimmen von wo bis wo die Y-Achse geht...
  void setXRange(float min_, float max_) {
    x_min = min_;
    x_max = max_;
  }  
  
  
  
  // Achsen 'ausklappen'
  void showAxes() {
    // Ausrechnen, wo sich das Koordinatensystem befindet in Pixelkoordinaten
    int xStelle = (int) map(0, x_min, x_max, 0, width); 
    int yStelle = (int) map(0, y_max, y_min, 0, height); 
    
    stroke(255);
    strokeWeight(2); 
    line(xStelle, 0, xStelle, height); // y-Achse
    line(0,yStelle,width,yStelle); // x-Achse
  }
  
  // Legende drucken
  void showLegend() {
    textSize(20);
    fill(g_,r_,b_);
    text("Temperatur", 625, 20);
    fill(0,205,255);
    text("Luftfeuchtigkeit", 625, 40);    
    fill(255,255,255);
    // Ausrechnen, wo sich das Koordinatensystem befindet in Pixelkoordinaten
    int xStelle = (int) map(0, x_min, x_max, 0, width); 
    int yStelle = (int) map(0, y_max, y_min, 0, height); 
    
    
    // X-Achse
    int n = floor(b/window); // die legende in n Teile gliedern für die Beschriftung
    //println("Legende hat (n) Markierungen auf der X-Achse. n = "+n);
    
    stroke(25);
    strokeWeight(1); 
    for (int i=0; i<=n; i++){
      line(window*i+(xStelle%window), 0, window*i+(xStelle%window), h);
    }
    stroke(255);
    strokeWeight(1); 
    for (int i=0; i<=n; i++){
      line(window*i+(xStelle%window), yStelle-5, window*i+(xStelle%window), yStelle+5);
    }
    
    // Y-Achse
    n = floor(h/window); // die legende in n Teile gliedern für die Beschriftung
    //println("Legende hat (n) Markierungen auf der Y-Achse. n = "+n);
    
    stroke(25);
    strokeWeight(1); 
    for (int i=0; i<=n; i++){
      line(0, i*window+(yStelle%window), b, i*window+(yStelle%window));
    }
    stroke(255);
    strokeWeight(1); 
    for (int i=0; i<=n; i++){
      line(xStelle-5, i*window+(yStelle%window), xStelle+5, i*window+(yStelle%window));
    }
    
    //float yRangePixels = y_max-y_min;
    
    for (int i=0; i<=floor(h/window); i++){  
      textSize(10);
      text((int)map(i,0,n,y_max, y_min), xStelle-30, i*window+(yStelle%window)+5);
    }  
    
    int time_ = 0;
    for (int i=0; i<=floor(b/window); i++){   
      textSize(10);
    //  text((int)map(i,0,floor(b/window),x_min, x_max), i*window+(xStelle%window)-5, yStelle+20);
      if (i>1) {
        text(time_+11 +" Uhr", i*window+(xStelle%window)-5, yStelle+20);
      }
      if (i==13) 
        time_ = time_-24;
      time_ = time_ +1;
    }  
  }
  
  
  void punkteLoeschen() {
    background(0,50,20);
    this.showLegend();
    this.showAxes();
  }
  
  
  // einen Punkt ins Koordinatensystem plotten
  void druckePunkt(float xPos, float yPos) {  
    // Positionen umrechnen bevor sie auf die Leinwand gezeichnet werden koennen, 
    // insofern der Wert ueberhaupt im darstellbaren Bereich liegt
    println("xPos: "+xPos+ " - yPos: "+ yPos);
    if (xPos>=x_min && xPos<=x_max && yPos>=y_min && yPos<=y_max) {
      //if (counter>= limit && limit!=0) {
      //  counter = 0;
      //  this.punkteLoeschen();
      //}
      
      xPos = ((int) map(xPos, x_min, x_max, 0, width)) ;
      yPos = (height - (int) map(yPos, y_min, y_max, 0, height)) ;
      println("xPos_new: " +xPos+ " - yPos_new: "+ yPos);
      stroke(r_,g_,b_);
      strokeWeight(4); 
      point(xPos, yPos-1);
      counter++;
      
    }else{
      if(xPos>=x_max) this.punkteLoeschen();    //löscht immer wenn Daten nach rechts rauswandern
    }
  }
 
  
  // einen Punkt ins Koordinatensystem plotten
  void druckePunkt(int xPos, int yPos) {  
    this.druckePunkt((float) xPos, (float) yPos);
  }  
  
 
  void punktFarbe(int r, int g, int b) {
    r_ = r;
    g_ = g;
    b_ = b;
  }
  
  // wieviele Punkte kommen ins Koordinatensystem bis es geloescht wird?
  void setLimit(int l) {
    limit = l;
  }
  
}// Ende der Klasse fuer das Koordinatensystem
